#include <fstream>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "psExporter.h"

/** Constructor for the Postscript export filter
 */
psExporter::psExporter(char *what_file, nodes *what_nodes) :
    s_filename(what_file), my_nodes(what_nodes),
    margin(30), scale_factor(1.7) {
}


/** Perform the export 
 */
void psExporter::do_export() {
    std::ofstream outfile;

    outfile.open(s_filename.c_str());

    find_bounds();
    write_headers(outfile);
    write_edges(outfile);
    write_nodes(outfile);

    outfile.close();
}


/** Write the standard Postscript headers
 */
void psExporter::write_headers(std::ofstream &outfile) {
    outfile << "\%!PS-Adobe-1.0" << std::endl;
    outfile << "%%BoundingBox: 0 0 " << (translatexcoord((double) biggest_x) + 2 * margin);
    outfile << " " << (translateycoord((double) biggest_y) + 2 * margin) << std::endl;
    outfile << "%%DocumentFonts: Helvetica" << std::endl;
    outfile << "%%Pages: 1" << std::endl;
}


/** Find the minimum and maximum offsets for the nodes
 */
void psExporter::find_bounds() {
    smallest_x = smallest_y =  100000;
    biggest_x  = biggest_y  = -100000;
    node *temp_node;

    for (int temp = 0; temp < my_nodes->number_of_nodes(); temp++)
    {
        temp_node = my_nodes->get_node(temp);
        if (temp_node->x_pos < smallest_x) smallest_x = (int) temp_node->x_pos;
        if (temp_node->y_pos < smallest_y) smallest_y = (int) temp_node->y_pos;
        if (temp_node->x_pos > biggest_x)  biggest_x  = (int) temp_node->x_pos;
        if (temp_node->y_pos > biggest_y)  biggest_y  = (int) temp_node->y_pos;
    }
}


/** Dump out the nodes to the file 
 */
void psExporter::write_nodes(std::ofstream &outfile) {
    node *temp_node;

    for (int temp = 0; temp < my_nodes->number_of_nodes(); temp++) {
        temp_node = my_nodes->get_node(temp);
        write_node(temp_node, outfile);
    }
}


/** Dump out a single node to the file
 */
void psExporter::write_node(node *this_node, std::ofstream &outfile) {
    outfile << "/Helvetica findfont 10 scalefont setfont" << std::endl;
    outfile << "(" << this_node->get_name() << ") stringwidth ";
    outfile << "/textywidth exch def /textxwidth exch def ";
    outfile << "newpath" << std::endl;
    outfile << translatexcoord(this_node->x_pos) << " textxwidth 2 div sub 2 sub ";
    outfile << translateycoord(this_node->y_pos) << " 14 2 div add moveto ";
    outfile << "textxwidth 4 add 0 rlineto 0 14 -1 mul rlineto ";
    outfile << "textxwidth 4 add -1 mul 0 rlineto closepath" << std::endl;
    outfile << "gsave 1 setgray fill grestore stroke /Helvetica findfont 10 ";
    outfile << "scalefont setfont" << std::endl;
    outfile << translatexcoord(this_node->x_pos) << " textxwidth 2 div sub ";
    outfile << translateycoord(this_node->y_pos) << " 4 sub moveto" << std::endl;
    outfile << "(" << this_node->get_name() << ") show" << std::endl;
}


/** Dump out the edges to the file 
 */
void psExporter::write_edges(std::ofstream &outfile) {
    node *temp_node;
    node *temp_node2;
    for (int temp = 0; temp < my_nodes->number_of_nodes(); temp++) {
        temp_node = my_nodes->get_node(temp);
        for (int temp2 = 0; temp2 < temp_node->number_of_connections; temp2++) {
            temp_node2 = temp_node->connecting_nodes[temp2];
            write_edge(temp_node, temp_node2, outfile);
        }
    }
}


/** Dump out a single edge to the file
 */
void psExporter::write_edge(node *from_node, node *to_node, std::ofstream &outfile) {
    outfile << "newpath" << std::endl;
    outfile << translatexcoord(from_node->x_pos) << " " << translateycoord(from_node->y_pos);
    outfile << " moveto " << translatexcoord(to_node->x_pos) << " ";
    outfile << translateycoord(to_node->y_pos) << " lineto closepath stroke" << std::endl;
}


// Utility methods

int psExporter::translatexcoord(double x) {
    return (int) ((x - smallest_x) * scale_factor) + margin * 2;
}


int psExporter::translateycoord(double y) {
    return (int) ((y - smallest_y) * scale_factor) + margin;
}

